from tempfile import mkstemp
from shutil import move
from mmc.support.mmctools import shLaunch
from mmc.support.config import PluginConfig
from ConfigParser import NoSectionError, NoOptionError
from mmc.plugins.services import restart

import re
import grp
import pwd
import shutil
import os

# Read ini configuration
class FtpConfig(PluginConfig):
    def readConf(self):
        PluginConfig.readConf(self)

        try: self.ftpConfPath = self.get("ftp", "ftpConfPath")
        except (NoSectionError, NoOptionError): self.ftpConfPath = "/etc/proftpd.conf"

        try: self.annonConfPath = self.get("ftp", "annonConfPath")
        except (NoSectionError, NoOptionError): self.annonConfPath = "/etc/proftpd.d/annon.conf"

        try: self.ftpInit = self.get("ftp", "ftpInit")
        except (NoSectionError, NoOptionError): self.ftpInit = "/etc/init.d/proftpd"

class Ftp(object):
    config = FtpConfig("ftp")

    # set global FTP configurations if the parameter do not exists on file, append it
    def setConf(self, param, key):

        # php can pass or not some key values, if not do nothing
        if key:
            if(param.startswith("#")):  # to enable/disable this kind of parameter has to uncomment or comment then do this
                param = param[1:]  # remove flag '#'
                rt1 = shLaunch("grep '^#" + param + "' " + Ftp.config.ftpConfPath)
                rt2 = shLaunch("grep '^" + param + "' " + Ftp.config.ftpConfPath)
                if(rt1.exitCode == 0):
                    shLaunch("sed -i 's,^#" + param + ".*," + param + " " + key + ",g' " + Ftp.config.ftpConfPath)
                elif(rt2.exitCode == 0):
                    shLaunch("sed -i 's,^" + param + ".*,#" + param + " " + key + ",g' " + Ftp.config.ftpConfPath)
                else:  # do not exists, create it
                    self.insertConf(Ftp.config.ftpConfPath, '^' + param, ' ' + key + '\n')
                restart("proftpd")
                return 0

            rt = shLaunch("cat " + Ftp.config.ftpConfPath + " | grep '" + param + "' ")
            if(rt.exitCode != 0):
                self.insertConf(Ftp.config.ftpConfPath, '^' + param, '\t\t\t' + key + '\n')
            else:
                self.replaceConf(Ftp.config.ftpConfPath, '^' + param, '\t\t\t' + key + '\n')
            restart("proftpd")
            return 0

    def setAnonPath(self, pth, maxuser):
        # return 0 = ok, return 1 if the dir do not exists in the path variable
        if(pth != 'remove'):
            if not os.path.exists(pth):
                return 1

            if os.path.exists(Ftp.config.annonConfPath):
                # If the file exists, just update the info.
                self.replaceConf(Ftp.config.annonConfPath, '^<Anonymous', ' ' + pth + ' >\n')
                self.replaceConf(Ftp.config.annonConfPath, '\tMaxClients', ' ' + maxuser + ' "Sorry, max %m users -- try again later"\n')

            else:
                # If the file does not exists, create a new one
                self.createAnnon(pth, maxuser)

        # Remove annon conf file
        else:
            if os.path.exists(Ftp.config.annonConfPath):
                os.remove(Ftp.config.annonConfPath)

        restart("proftpd")
        return 0

    def getAnonPath(self):
        if not os.path.exists(Ftp.config.annonConfPath):
            return "/var/ftp"

        f = open(Ftp.config.annonConfPath, 'r')
        searchObj = re.search(r'<Anonymous (.*) >', f.read())
        f.close()

        if(searchObj.group(1) == ""):
            return "/var/ftp"
        else:
            return searchObj.group(1)

    def getAnonUsers(self):
        if not os.path.exists(Ftp.config.annonConfPath):
            return "/var/ftp"

        f = open(Ftp.config.annonConfPath, 'r')
        searchObj = re.search(r'\s*MaxClients\s*(\S*)\s*', f.read())
        f.close()

        if(searchObj.group(1) == ""):
            return "0"
        else:
            if searchObj.group(1) == 'none':
                return 0
            else:
                return searchObj.group(1)

    # test if public share is enabled
    def isEnabled(self, pattern, place):

        # Decides which config file we are working with
        if place == "base":
            path = Ftp.config.ftpConfPath
        elif place == "annon":
            path = Ftp.config.annonConfPath

        # Check if file exists
        if not os.path.exists(path):
            return 0

        # Search the pattern
        f = open(path, 'r')
        searchObj = re.search(r'\s*' + pattern +'\s*(.*)', f.read())
        f.close()

        if(searchObj.group(1) == ""):
            return "0"  # Cant find the config, then no.
        else:
            if searchObj.group(1).lower() == 'off':
                return 0
            else:
                return 1

    # replace some configuration making temp file and replacing what you want to temp file
    def replaceConf(self, file_path, pattern, key):
        fh, abs_path = mkstemp()
        new_file = open(abs_path, 'w')
        old_file = open(file_path)
        for line in old_file:
            if(re.match(pattern, line) is not None):
                new_file.write(pattern.translate(None, '^&$?*') + key)  # remove the special symbols for regex pattern
            else:
                new_file.write(line)
        new_file.close()
        os.close(fh)
        old_file.close()
        os.remove(file_path)
        move(abs_path, file_path)

    # or just append the new configuration to the end of the file
    def insertConf(self, file_path, pattern, key):
        fh = open(file_path, 'a')
        fh.write(pattern.translate(None, '^&$?*') + key)  # remove the special symbols for regex pattern
        fh.close()

    # input regex to find in file, return value of the param in the config file
    def getConf(self, param):
        param = param.translate(None, '^$?*')
        ret = shLaunch("grep '" + param + "' " + self.config.ftpConfPath + " | sed 's/" + param + "[\\t| |\\n]*//g'")
        ret.out = ret.out.replace('\n', '')
        ret.out = ret.out.replace('"', '')
        return ret.out

    # enable ftp share for each user, return 2 = user do not exists, 1 = file exists, 0 = everything ok
    def ftpEnableUser(self, name):
        ret = os.path.isdir('/home/' + name)
        if not ret:
            return 2
        ret = os.path.isdir("/home/" + name + "/share_ftp")
        if not ret:
            try:
                pwd.getpwnam(name)
            except KeyError:
                return 2
            os.mkdir('/home/' + name + '/share_ftp')
            uid = pwd.getpwnam(name).pw_uid
            gid = grp.getgrnam("ftp").gr_gid
            os.chown("/home/" + name + "/share_ftp", uid, gid)
            os.chmod("/home/" + name + "/share_ftp", 0755)
            return 0
        else:
            return 1

    def ftpDisableUser(self, name):
        ret = os.path.isdir('/home/' + name)
        if not ret:
            return 2
        ret = os.path.isdir("/home/" + name + "/share_ftp")
        if ret:
            shutil.rmtree('/home/' + name + '/share_ftp')
            return 0
        else:
            return 1

    # get all users with ftp share anabled
    def ftpGetUsers(self):
        ret = shLaunch("find /home -maxdepth 2 -name 'share_ftp' -type d | cut -d / -f 3")
        users = ret.out
        users = users.split('\n')
        users = [x for x in users if x]
        if len(users) == 0:
            return 0
        else:
            return users

    # Create a new annon config file
    def createAnnon(self, pth, maxuser):
        fh = open(Ftp.config.annonConfPath, 'w')
        fh.write('<Anonymous ' + pth + ' >\n')
        fh.write('\t# Allow logins if they are disabled above.\n')
        fh.write('\t<Limit LOGIN>\n')
        fh.write('\t\tAllowAll\n')
        fh.write('\t</Limit>\n')
        fh.write('\n')
        fh.write('\t# Maximum clients with message\n')
        fh.write('\tMaxClients ' + maxuser + ' "Sorry, max %m users -- try again later"\n')
        fh.write('\n')
        fh.write('\tUser\t\t\tftp\n')
        fh.write('\tGroup\t\t\tftp\n')
        fh.write('\t# We want clients to be able to login with "anonymous" as well as "ftp"\n')
        fh.write('\tUserAlias\tanonymous\tftp\n')
        fh.write('\tRequireValidShell\t\t\toff\n')
        fh.write('\n')
        fh.write('\t# Limit WRITE everywhere in the anonymous chroot\n')
        fh.write('\t<Limit WRITE>\n')
        fh.write('\t\tDenyAll\n')
        fh.write('\t</Limit>\n')
        fh.write('</Anonymous>\n')
        fh.close()
